<?php
namespace Woocommerce\Moip\Fields\Payment_Split;

if ( ! function_exists( 'add_action' ) ) {
	exit( 0 );
}

use Woocommerce\Moip\Core;
use Woocommerce\Moip\Helper\Utils;
use Woocommerce\Moip\Model\Custom_Gateway;
use Woocommerce\Moip\Model\Setting;

//Views
use Woocommerce\Moip\View;

class Config
{
    public static function section_payment_split()
	{
		return array(
			'title'       => __( '', Core::TEXTDOMAIN ),
			'type'        => 'title',
			'description' => sprintf(
				'%s',
				self::split_message_soon()
			),
		);
    }

    public static function split_message_soon()
	{
		$model      = new Custom_Gateway();
		$authorized = strtoupper( $model->settings->authorize_mode );
		$message    = __( 'Is authorized via <strong>'.$authorized.'</strong>!', Core::TEXTDOMAIN );

		if ( empty( $authorized ) ) {
			$message    = __( 'NOT AUTHORIZED :(', Core::TEXTDOMAIN );
		}

        return $message;
    }

    public static function wirecard_checkbox_payment_split()
	{
		return array(
			'title'       => __( 'Payment Split', Core::TEXTDOMAIN ),
			'type'        => 'checkbox',
			'label'       => __( 'Enable payment split', Core::TEXTDOMAIN ),
			'default'     => 'no',
			'description' => __( 'After generating the app you need to enable split to work.', Core::TEXTDOMAIN ),
			'desc_tip'    => false,
			'custom_attributes' => array(
				'data-field' => 'render-wirecard-payment-split',
			),
		);
	}

	public static function split_shipping_method()
	{
		return array(
			'type'              => 'select',
			'title'             => __( 'Shipping Method', Core::TEXTDOMAIN ),
			'class'             => 'wc-enhanced-select',
			'default'           => 'split_shipping_vendor',
			'description'       => __( 'Choose who will pay the split payment shipping fee. By default it will be the Vendor.', Core::TEXTDOMAIN ),
			'desc_tip'          => false,
			'custom_attributes' => array(
				'data-element'  => 'split-shipping-method',
				'data-action'   => 'split-shipping-method-type',
			),
			'options' => array(
				'split_shipping_admin'  => __( 'Administrator', Core::TEXTDOMAIN ),
				'split_shipping_vendor' => __( 'Vendor', Core::TEXTDOMAIN ),
			),
		);
	}

	public static function marketplace_options_type()
	{
		$marketplace_default = 'empty_marketplace'; 
		/*Dokan Lite Class*/
		if ( class_exists('WeDevs_Dokan') ){
			$marketplace_default = 'dokan_marketplace';
		}
		/*WCFM Class*/
		if ( class_exists('WCFMmp') ){
			$marketplace_default = 'wcfm_marketplace';
		}

		return array(
			'type'              => 'select',
			'title'             => __( 'Marketplace Type', Core::TEXTDOMAIN ),
			'class'             => 'wc-enhanced-select',
			'default'           => $marketplace_default,
			'description'       => __( 'Select the marketplace according to the installed plugin, Dokan or WCFM.' , Core::TEXTDOMAIN ),
			'custom_attributes' => array(
				'data-element'  => 'marketplace',
				'data-action'   => 'marketplace-type',
			),
			'options' => array(
				'empty_marketplace' => __( 'Select your Marketplace', Core::TEXTDOMAIN ),
				'dokan_marketplace' => __( 'Dokan', Core::TEXTDOMAIN ),
				'wcfm_marketplace'  => __( 'WCFM', Core::TEXTDOMAIN ),
			),
		);
	}

    public static function wirecard_split_name()
	{
		return array(
			'title'       => __( 'Wirecard Split Name', Core::TEXTDOMAIN ),
			'type'        => 'text',
			'desc_tip'    => false,
			'default'     => '',
			'placeholder' => __( 'Wirecard Brazil Official Split', Core::TEXTDOMAIN ),
			'description' => __( 'Enter the name of the payment split for your store. Default: Wirecard Brazil Official Split' , Core::TEXTDOMAIN ),
			'custom_attributes' => array(
				'maxlength' => 40
			),
		);
	}

    public static function wirecard_split_description()
	{
		return array(
			'title'       => __( 'Wirecard Split Description', Core::TEXTDOMAIN ),
			'type'        => 'text',
			'desc_tip'    => false,
			'default'     => '',
			'placeholder' => __( 'Wirecard Brazil Official Payment Split', Core::TEXTDOMAIN ),
			'description' => __( 'Enter the description of the payment split for your store. Default: Wirecard Brazil Official Payment Split' , Core::TEXTDOMAIN ),
			'custom_attributes' => array(
				'maxlength' => 50
			),
		);
	}

	public static function wirecard_manual_app_credentials()
	{
		return array(
			'title'       => __( 'APP credentials', Core::TEXTDOMAIN ),
			'type'        => 'title',
			'desc_tip'    => false
		);
	}

	public static function wirecard_manual_token_account()
	{
		return array(
			'title'       => __( 'Wirecard Token', Core::TEXTDOMAIN ),
			'type'        => 'text',
			'desc_tip'    => false,
			'default'     => '',
			'custom_attributes' => array(
				'data-action'    => 'wirecard-manual-token',
				'data-element'   => 'validate',
				'data-error-msg' => __( 'This field is required.', Core::TEXTDOMAIN ),
			),
			'description' => sprintf(
				__( 'Copy and paste wirecard token, where to find: %s' , Core::TEXTDOMAIN ),
				self::get_wirecard_manual_token()
			)
		);
	}

	public static function get_wirecard_manual_token()
	{
		return '<a href="' . Utils::get_url_endpoint_connect() . '">'
			. __( 'My account &gt; Settings &gt; Access Keys', Core::TEXTDOMAIN ) . '</a> '
			. __( 'In Authentication Key copy TOKEN.', Core::TEXTDOMAIN );
	}

	public static function wirecard_manual_key_account()
	{
		return array(
			'title'       => __( 'Wirecard Key', Core::TEXTDOMAIN ),
			'type'        => 'text',
			'desc_tip'    => false,
			'default'     => '',
			'custom_attributes' => array(
				'data-action'    => 'wirecard-manual-key',
				'data-element'   => 'validate',
				'data-error-msg' => __( 'This field is required.', Core::TEXTDOMAIN ),
			),
			'description' => sprintf(
				__( 'Copy and paste wirecard key, where to find: %s' , Core::TEXTDOMAIN ),
				self::get_wirecard_manual_key()
			)
		);
	}

	public static function get_wirecard_manual_key()
	{
		return '<a href="' . Utils::get_url_endpoint_connect() . '">'
			. __( 'My account &gt; Settings &gt; Access Keys', Core::TEXTDOMAIN ) . '</a> '
			. __( 'In Authentication Key click: [click here to show key] copy KEY.', Core::TEXTDOMAIN );
    }

    public static function section_wirecard_app()
	{
		return array(
			'title' => __( 'Wirecard APP', Core::TEXTDOMAIN ),
            'type'  => 'title',
            'description' => sprintf(
				'%s',
				self::generate_wirecard_app()
			),
		);
    }

	public static function generate_wirecard_app()
	{
		$button_notification = View\Payment_Splits::render_button_wirecard_split();

		return $button_notification;
    }

    public static function section_app_info()
	{
		return array(
			'title'       => __( 'App Info', Core::TEXTDOMAIN ),
			'type'        => 'title',
			'description' => self::get_app_info()
		);
	}

	public static function get_app_info()
	{
		$model      = new Custom_Gateway();
		$all_token  = '';

		if ( $model->settings->authorize_data ) {
			$all_token .= View\Payment_Splits::render_info_wirecard_app( $model->settings );
		}

		return $all_token;
	}

	public static function section_title_marketplace()
	{
		return array(
			'title'       => __( '', Core::TEXTDOMAIN ),
			'type'        => 'title',
			'description' => sprintf(
				'%s',
				self::marketplace_message_soon()
			),
		);
    }

    public static function marketplace_message_soon()
	{
		$dokan_plugin = 'https://wordpress.org/plugins/dokan-lite/';
		$wcfm_plugin  = 'https://wordpress.org/plugins/wc-multivendor-marketplace/';

        return sprintf(
            '%s
            <a href="%s">%s</a> %s <a href="%s">%s</a> %s',
            __( 'To work with the payment split, you need to install and activate the', Core::TEXTDOMAIN ),
            esc_url( $dokan_plugin ),
			__( 'Dokan Lite', Core::TEXTDOMAIN ),
			__( 'or', Core::TEXTDOMAIN ),
			esc_url( $wcfm_plugin ),
			__( 'WCFM Marketplace', Core::TEXTDOMAIN ),
            __( 'marketplace plugin.', Core::TEXTDOMAIN )
        );
	}

	public static function section_title_payment_type()
	{
		return array(
			'title'       => __( '', Core::TEXTDOMAIN ),
			'type'        => 'title',
			'description' => sprintf(
				'%s',
				self::payment_type_message_soon()
			),
		);
    }

    public static function payment_type_message_soon()
	{
		$url_plugin = Utils::get_admin_url( 'admin.php' ) . '?page=wc-settings&tab=checkout&section=woo-moip-official&wirecard-tab=wbo-general';

        return sprintf(
            '%s
            <a href="%s">%s</a>',
            __( 'To work with the payment split you need to change the payment type to Transparent Checkout, ', Core::TEXTDOMAIN ),
            esc_url( $url_plugin ),
            __( 'CHANGE NOW', Core::TEXTDOMAIN )
        );
    }

	public static function get_fields()
	{
		$setting = Setting::get_instance();

		/*Dokan Lite Class and WCFM Class*/
		if ( !class_exists('WeDevs_Dokan') && !class_exists('WCFMmp') ){
			return array( 'section_title_marketplace' => self::section_title_marketplace() );
		}

		/*Checkout Transparente*/
		if ( !$setting->is_checkout_transparent() ) {
			return array( 'section_title_payment_type' => self::section_title_payment_type() );
		}

		return array(
			'section_payment_split'       => self::section_payment_split(),
            'wirecard_payment_split'      => self::wirecard_checkbox_payment_split(),
			'marketplace_options_type'    => self::marketplace_options_type(),
			'split_shipping_method'       => self::split_shipping_method(),
            'wirecard_split_name'         => self::wirecard_split_name(),
			'wirecard_split_description'  => self::wirecard_split_description(),
			'wirecard_manual_credentials' => self::wirecard_manual_app_credentials(),
			'wirecard_manual_token'       => self::wirecard_manual_token_account(),
			'wirecard_manual_key'         => self::wirecard_manual_key_account(),
			'section_wirecard_app'        => self::section_wirecard_app(),
			'section_app_info'            => self::section_app_info()
        );
	}
}
