<?php
if ( ! function_exists( 'add_action' ) ) {
	exit(0);
}

use Woocommerce\Moip\Core;
use Woocommerce\Moip\Model\Setting;
use Woocommerce\Moip\Model\Marketplace;

$model            = new Marketplace();
$settings         = Setting::get_instance();
$current_user     = wp_get_current_user();
$admin_user       = current_user_can( 'administrator' );
$wirecard_user_id = get_user_meta( $current_user->ID, '_wirecard_account_id', true );
$status           = __( '<strong>A loja ainda não está autorizada!<strong>', Core::TEXTDOMAIN );
$url              = '#';

if ( $settings->authorize_mode == 'sandbox' ) {
	$status = __( 'A loja está autorizada como <strong>SANDBOX</strong>.', Core::TEXTDOMAIN );
	$url    = 'https://bem-vindo-sandbox.wirecard.com.br/';
}

if ( $settings->authorize_mode == 'production' ) {
	$status = __( 'A loja está autorizada como <strong>PRODUCTION</strong>.', Core::TEXTDOMAIN );
	$url    = 'https://bem-vindo.wirecard.com.br/';
}

?>
<div class="collapse wcfm-collapse" id="wcfm_wirecard_settings">
  <div class="wcfm-page-headig">
		<span class="wcfmfa fa-retweet"></span>
		<span class="wcfm-page-heading-text"><?php _e( 'Wirecard Settings', Core::TEXTDOMAIN ); ?></span>
		<?php do_action( 'wcfm_page_heading' ); ?>
	</div>
	<div class="wcfm-collapse-content">
	  <div id="wcfm_page_load"></div>

	  <div class="wcfm-container wcfm-top-element-container">
			<h2><?php _e( 'Wirecard Settings - Payment Split', Core::TEXTDOMAIN ) ?></h2>
			<div class="wcfm-clearfix"></div>
		</div>
	  <div class="wcfm-clearfix"></div><br />
			<div class="wcfm-container">
			<?php if ( $admin_user ) : ?>
                <p class="description">
                    <?php echo __( 'You are the site administrator you cannot link as a shopkeeper to perform tests!', Core::TEXTDOMAIN ); ?>
                </p>
            <?php endif; ?>
            
            <?php if ( !$admin_user && strlen( $wirecard_user_id ) > 0 ) : ?>
            <div class="wcfm-wirecard-container">
                <p class="description">
                    <?php echo $status; ?>
                </p>
                <p class="description">
                    <?php echo __( 'Your account is already linked to Wirecard. If you need you can authorize again by clicking the button below.', Core::TEXTDOMAIN ); ?>
                </p>
                <a class="wbo_reauthorize_client text_tip" href="<?php echo esc_url( $model->get_url_connect() ); ?>">
                    <button type="button" class="reauthorize-button" style="border: solid 1px #333333;border-radius: 3px;">
					<span style="margin-right:7px;" class="wcfmfa fa-user-plus"></span>
                        <?php echo __( 'Authorize Again', Core::TEXTDOMAIN ); ?>
                    </button>
                </a>  
                <p class="description ">
                    <?php echo __( 'Click the button to sign in to your Wirecard account again if necessary.', Core::TEXTDOMAIN ); ?>
                </p>
            </div>
            <?php endif; ?>

            <?php if ( !$admin_user && strlen( $wirecard_user_id ) == 0 ) : ?>
                <div class="wcfm-wirecard-container">
					<p class="description">
						<?php echo $status; ?>
					</p>
                    <div class="wcfm-page-help" style="margin-bottom:20px;">
                        <?php _e( 'Before authorizing you must have a Wirecard account.', Core::TEXTDOMAIN ); ?>
						<?php
							echo sprintf(
								'<a href="%s" target="__blank">%s</a>',
								esc_url( $url ),
								__( 'Create an account', Core::TEXTDOMAIN )
							); 
						?>
                    </div>
					<a class="wbo_authorize_client text_tip" href="<?php echo esc_url( $model->get_url_connect() ); ?>">
                        <button type="button" class="authorize-button" style="border: solid 1px #333333;border-radius: 3px;">
							<span style="margin-right:7px;" class="wcfmfa fa-user-plus"></span>
                            <?php echo __( 'Authorize', Core::TEXTDOMAIN ); ?>
                        </button>
                    </a>    
                    <p class="description ">
                        <?php echo __( 'Click the button to sign in to your Wirecard account.', Core::TEXTDOMAIN ); ?>
                    </p>
                </div>
            <?php endif; ?> 
			</div>
			<div class="wcfm-clearfix"></div>
	</div>
</div>
